/** Class to help installing BSF4Rexx, will output all <code>java.lang.System.getProperties()</code>
 *  as a &quot;<code>propertyName=[value]</code>&quote; pair, each delimited with
 *  &quot;<code>\\n</code>&quot;. This allows RxQueue'ing the output into any Rexx program for further
 *  analysis.
 *
 * <p>
 *
 * <pre>------------------------ Apache Version 2.0 license -------------------------
 *    Copyright (C) 2005-2007 Rony G. Flatscher
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 * ----------------------------------------------------------------------------- </pre>
 *
 *
 * @author Rony G. Flatscher (c) 2005, 2006, 2007
 * @version 1.0.1
 *
 */

 /*
    changes:
        2006-08-01: now tests for presence of "org.apache.commons.logging.LogFactory"; returning
                    "rgf.commons.logging.available=[1]" if loadable, else
                    "rgf.commons.logging.available=[0]"
        2007-01-28: using Thread.currentThread().getContextClassLoader().loadClass(...) instead of Class.forName(...),
                    now returns information on availability of BSF and BSF4Rexx, e.g. (if not available,
                    version string is empty string):

                        org.apache.bsf.BSFManager.available=[1]
                        org.apache.bsf.BSFManager.version=[242.20070128]
                        org.rexxla.bsf.engines.rexx.RexxEngine.available=[0]
                        org.rexxla.bsf.engines.rexx.RexxEngine.version=[]

                    if argument is given, returns only what java.lang.System.getProperty(arg) returns
                    (does not list all properties!)

   last change: $Revision: 168 $ $Author: rony $ $Date: 2006-08-01 21:42:20 +0200 (Tue, 01 Aug 2006) $
 */

public class JavaInfo4BSF4RexxInstallation
{
    public static void main (String args[])
    {
        if (args.length==0)
        {
            System.out.println("--> no args!");
            showAllProperties();
        }
        else
        {
            // String val=System.getProperty(args[0]);
            // System.out.println("args[0]=["+args[0]+"] val=["+val+"]");
            // System.out.print(val==null ? "" : val);
            System.out.println(System.getProperty(args[0]));
        }

        // showSomeProperties();
        System.exit(0);
    }

    static char [] letters={ '0', '1', '2', '3', '4', '5', '6', '7',
                             '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };

    static void showSomeProperties()
    {
        StringBuffer sb=new StringBuffer();    // create a StringBuffer
        String       nl=System.getProperty("line.separator");

        sb=sb.append("file.separator="             ).append("[").append(  nullString(System.getProperty("file.separator"             ))).append("]").append(nl);
        sb=sb.append("path.separator="             ).append("[").append(  nullString(System.getProperty("path.separator"             ))).append("]").append(nl);
        sb=sb.append("line.separator="             ).append("[").append(  nullString(System.getProperty("line.separator"             ))).append("]").append(nl);

        sb=sb.append("java.class.version="         ).append("[").append(  nullString(System.getProperty("java.class.version"         ))).append("]").append(nl);
        sb=sb.append("java.ext.dirs="              ).append("[").append(  nullString(System.getProperty("java.ext.dirs"              ))).append("]").append(nl);
        sb=sb.append("java.home="                  ).append("[").append(  nullString(System.getProperty("java.home"                  ))).append("]").append(nl);
        sb=sb.append("java.specification.version=" ).append("[").append(  nullString(System.getProperty("java.specification.version" ))).append("]").append(nl);
        sb=sb.append("java.vendor.url"             ).append("[").append(  nullString(System.getProperty("java.vendor.url"            ))).append("]").append(nl);
        sb=sb.append("java.vendor="                ).append("[").append(  nullString(System.getProperty("java.vendor"                ))).append("]").append(nl);
        sb=sb.append("java.version="               ).append("[").append(  nullString(System.getProperty("java.version"               ))).append("]").append(nl);

        sb=sb.append("os.arch="                    ).append("[").append(  nullString(System.getProperty("os.arch"                    ))).append("]").append(nl);
        sb=sb.append("os.name="                    ).append("[").append(  nullString(System.getProperty("os.name"                    ))).append("]").append(nl);
        sb=sb.append("os.version="                 ).append("[").append(  nullString(System.getProperty("os.version"                 ))).append("]").append(nl);

        sb=sb.append("user.name="                  ).append("[").append(  nullString(System.getProperty("user.name"                  ))).append("]").append(nl);
        sb=sb.append("user.home="                  ).append("[").append(  nullString(System.getProperty("user.home"                  ))).append("]").append(nl);
        sb=sb.append("user.dir="                   ).append("[").append(  nullString(System.getProperty("user.dir"                   ))).append("]").append(nl);

        sb=sb.append("java.library.path="          ).append("[").append(  nullString(System.getProperty("java.library.path"          ))).append("]").append(nl);
        sb=sb.append("java.class.path="            ).append("[").append(  nullString(System.getProperty("java.class.path"            ))).append("]").append(nl);

            // test for availability of org.apache.commons.logging
        sb=sb.append( get_commons_logging_availability()).append(nl);

        sb=sb.append( get_org_apache_bsf_availability(nl)).append(nl);

        System.out.println(sb);
    }

    static void showAllProperties()
    {
        java.util.Enumeration prop_names=System.getProperties().propertyNames();   // get all the property names

        StringBuffer sb=new StringBuffer();    // create a StringBuffer
        String       nl=System.getProperty("line.separator");
        String       key="";

        while (prop_names.hasMoreElements())
        {
            key=(String) prop_names.nextElement();
            sb=sb.append(key).append("=[").append( nullString(System.getProperty(key))).append("]").append(nl);;
        }

            // test for availability of org.apache.commons.logging
        sb=sb.append( get_commons_logging_availability             ()  ).append(nl);
        sb=sb.append( get_org_apache_bsf_availability              (nl) ).append(nl);
        sb=sb.append( get_org_rexxla_bsf_engines_rexx_availability (nl) ).append(nl);

        System.out.println(sb);
    }

    static String nullString(String o)      // if null, return empty String instead
    {
        if (o==null) return "";

        // check for chars < x20, encode them
        StringBuffer sb=new StringBuffer();
        char         ch=0;
        for (int i=0; i<o.length(); i++)
        {
            if (o.charAt(i)>31)
               continue;
            // o.k., char < 0x20: we need to transform this string into a hex-string "0xHEXDATA"
            sb.append("0x");
            for (int k=0; k<o.length(); k++)
            {
                ch=o.charAt(k);
                sb.append(letters[(ch >>> 16)]);
                sb.append(letters[(ch & 0x0f) ]);
            }
            return sb.toString();
        }

        return o;
    }



    static String get_commons_logging_availability()
    {
        String kw="org.apache.commons.logging.LogFactory=[";    // define keyword and leadin
        try
        {
            // try to load the class dynamically
            // Class lf=Class.forName("org.apache.commons.logging.LogFactory");

            Class lf=Thread.currentThread().getContextClassLoader().loadClass("org.apache.commons.logging.LogFactory");
        }
        catch (ClassNotFoundException e)
        {
            return kw+"0]"; // return Rexx character for .false
        }
        return kw+"1]";     // return Rexx character for .true

    }



    // use reflection to deal with this class (could be absent)
    static String get_org_apache_bsf_availability(String nl)
    {
        String clzName="org.apache.bsf.BSFManager";
        String kw=clzName+".available=[";   // define keyword and leadin
        Class  clz=null;

        try
        {
                // load class
            clz= Thread.currentThread().getContextClassLoader().loadClass(clzName);
            kw=kw+"1]"; // "1": Rexx character for .true
        }
        catch (ClassNotFoundException e)
        {
            kw=kw+"0]"; // "0": Rexx character for .false
        }

        kw=kw+nl+clzName+".version=[";     // now try to get version, if possible at all
        if (clz==null)      // no class, hence no value
        {
            kw=kw+"]";
        }
        else
        {
            try
            {
                    // get static field with Class object argument
                java.lang.reflect.Method m=clz.getMethod("getVersion", null);

                kw=kw+m.invoke(null, null).toString()+"]";   // a static field, hence "object" argument is ignored and may be null
            }
            catch (Exception e)
            {
                kw=kw+"]";
            }
        }

        return kw;     // return Rexx character for .true
    }




    // use reflection to deal with this class (could be absent)
    static String get_org_rexxla_bsf_engines_rexx_availability(String nl)
    {
        String clzName="org.rexxla.bsf.engines.rexx.RexxEngine";
        String kw=clzName+".available=[";   // define keyword and leadin
        Class  clz=null;


        try
        {
                // load class
            clz= Thread.currentThread().getContextClassLoader().loadClass(clzName);
            kw=kw+"1]"; // "1": Rexx character for .true
        }
        catch (ClassNotFoundException e)
        {
            kw=kw+"0]"; // "0": Rexx character for .false
        }

        kw=kw+nl+clzName+".version=[";     // now try to get version, if possible at all
        if (clz==null)      // no class, hence no value
        {
            kw=kw+"]";
        }
        else
        {
            try
            {
                    // get static field with Class object argument
                java.lang.reflect.Field f=clz.getField("version");
                kw=kw+f.get(null).toString()+"]";   // a static field, hence "object" argument is ignored and may be null
            }
            catch (Exception e)
            {
                kw=kw+"]";
            }
        }

        return kw;     // return Rexx character for .true
    }
}

